/*
 * UriShX breathing LED name card v1.0
 * Pulsing / breathing LEDs w/cap-sense for ATtiny13 https://hackaday.io/project/160831-cap-sense-name-card
 * 
 * Breathing LED function & explanation from https://sean.voisen.org/blog/2011/10/breathing-led-with-arduino/
 * NonBlockingBreathingLed 0.1 by Luca Soltoggio - 2015 http://www.arduinoelettronica.com/
 * 
 * PWM registers explanation from http://www.technoblogy.com/show?LE0
 * 
 * ADCTouch library: https://github.com/martin2250/ADCTouch
 * 
 * Released under GPL v.3 or later
 */
 
#include "pwm_var_table.h"
#include <ADCTouch.h>

#define ADC_PIN 3
#define LED1 0
#define LED2 1
#define THRESH 40  //Define threshold for ADCTouch
#define _DELAY  16   // delay between loops of PWM updating

unsigned int ref0;
byte timingVar = 0;
unsigned long breathe_time = millis();
bool isLit = false;
byte val = 0;


void setup()
{
  DDRB = 0x00;//set all pins as inputs. redundant.
  PORTB = 0x00;// sets all digital pins LOW
  DDRB |= (1 << DDB0) | (1 << DDB1); //B00000011;  // sets Arduino pins 1 to 7 as outputs, pin 0 as input
  TCCR0A = _BV(COM0A0) | _BV(COM0A1) | _BV(COM0B0) | _BV(COM0B1) | _BV(WGM00) | _BV(WGM01);//3<<COM0A0 | 3<<COM0B0 | 3<<WGM00;  // Configure counter/timer0 for fast PWM w/inverted input on PB0 and PB1

  delay(2500);//allow enough time for holder to place card so connecting battery won't affect ref0
  
  ref0 = ADCTouch.read(ADC_PIN, 512);    //create reference values to account for the capacitance of the pad

  //Confirm ref0 initialization
  if (ref0 > 0) PORTB |= (1 << PINB0) | (1 << PINB1);//LEDs off (inverted logic)
}

void loop()
{
  unsigned int touchSenseVal = ADCTouch.read(ADC_PIN, 32);   //no second parameter --> 100 samples
  touchSenseVal -= ref0;

  //touch state machine
  if (touchSenseVal > THRESH) {
    //if touch detected start pulsing
    nonBlockingBreath();
  } else if (isLit) {
    //if touch not currently detected but isLit flag on, dim
    for (byte i = val; i < 255; i++) {//inverted logic, 255 = led off
      OCR0A = i;
      OCR0B = i;
      delay(_DELAY * 2);
    }
    isLit = false;
  } else {
    val = 255;// PWM inverted logic, keep off
    OCR0A = val;// PWM
    OCR0B = val;// PWM
    timingVar = 0;
  }
}

void nonBlockingBreath()
{
  if ((breathe_time + _DELAY) < millis()) {
    isLit = true;
    breathe_time = millis();
    val = (byte)pgm_read_byte(pwmVal + timingVar);
    // this is the math function recreating the effect, replaced with PROGMEM values pre-caculated:
    // val = (exp(sin(timingVar/1000.0*PI*20)) - 0.36787944)*108.492; // pi*10 too slow
    OCR0A = val;// PWM
    OCR0B = val;// PWM
    timingVar++;
    //when timing variable grows beyond length of table, wrap around
    if (timingVar > 199) {
      timingVar = 0;
    }
  }
}

